! 	From: "COMPUTATIONAL PHYSICS, 2nd Ed" 
! 	by RH Landau, MJ Paez, and CC Bordeianu 
! 	Copyright Wiley - VCH, 2007.
! 	Electronic Materials copyright: R Landau, Oregon State Univ, 2007;
! 	MJ Paez, Univ Antioquia, 2007; and CC Bordeianu, Univ Bucharest, 2007.
! 	Supported by the US National Science Foundation				 
! 			
! 	 tune.f90: matrix algebra program to be tuned for performace			
!        CE: modified for use with SunStudio 12 routines
!           cup_time
!      and  system_clock

  
      Program tune
!     ------------

!      parameter (ldim = 2050)
      parameter (ldim = 6050)

      implicit double precision (a-h,o-z)

      dimension :: ham(ldim,ldim), coef(ldim), sigma(ldim)

!        for using timing routine cpu_time
      real :: timestart, timeend, cputimeused

!        for using timing routine system_clock
      integer :: istart, iend, irate
      real    :: elapsed_time

!        set up H and starting vector

      do i = 1, ldim
       do j = 1, ldim
        if ( abs(j - i).gt. 10) then
        ham(j, i) = 0.d0
        else 
        ham(j, i) = 0.3**abs(j - i)
        endif
       end do

       ham(i, i) = i
       coef(i) = 0.d0
      end do

       coef(1) = 1.d0

!        start iterating

      call cpu_time(timestart)
     
!      call system_clock(count=istart,count_rate=irate)

      err = 1.d0
      iter = 0 

 20      If (iter.lt.15 .and. err.gt.1.d-6) then
      iter = iter + 1

!        compute current energy & normalize

      ener = 0.d0 
      ovlp1 = 0.d0
      ovlp2 = 0.d0

      do i = 1, ldim-2,2 
      ovlp1 = ovlp1 + coef(i)*coef(i) 
      ovlp2 = ovlp2 + coef(i+1)*coef(i+1) 
      t1 = 0.d0
      t2 = 0.d0

       
       do j = 1, ldim
       t1 = t1 + coef(j)*ham(j, i)
       t2 = t2 + coef(j)*ham(j, i+1)
       end do

       sigma(i) = t1
       sigma(i+1) = t2

      ener = ener + coef(i)*t1 + coef(i+1)*t2
      end do 

      ovlp = ovlp1+ovlp2

      ener = ener/ovlp 

      do i = 1, ldim 
      coef(i) = coef(i)/sqrt(ovlp)
      sigma(i) = sigma(i)/sqrt(ovlp)
      end do

!        compute update and error norm
      err = 0.d0
       do i = 1, ldim 

       if (i == 1) goto 23 
       step = (sigma(i) - ener*coef(i))/(ener - ham(i, i))
       coef(i) = coef(i) + step
       err = err + step**2
 23    end do
      err = sqrt(err)

      call cpu_time(timeend)
      cputimeused = timeend-timestart

!      call system_clock(count=iend)
!      elapsed_time=real(iend-istart)/real(irate)


      write(6, '(1x, i2, 7f10.5)') iter, ener, err, coef(1)
      write (6,*) 'cpu time used:',cputimeused
!      write (6,*) 'cpu time used:',elapsed_time
      write (6,*)

      goto 20

      endif

!      call cpu_time(timeend)

      stop 
      end program tune

